@@ -11,20 +11,35 @@ from raven.utils import once
 special_logger_handlers = {}
 
 
+logger = logging.getLogger('raven')
+
+
+def event_payload_considered_equal(a, b):
+    return (
+        a['type'] == b['type'] and
+        a['level'] == b['level'] and
+        a['message'] == b['message'] and
+        a['category'] == b['category'] and
+        a['data'] == b['data']
+    )
+
+
 class BreadcrumbBuffer(object):
 
     def __init__(self, limit=100):
         self.buffer = []
         self.limit = limit
 
-    def record(self, type, timestamp=None, duration=None, level=None,
-               message=None, category=None, data=None, processor=None):
+    def record(self, timestamp=None, level=None, message=None,
+               category=None, data=None, type=None, processor=None):
+        if not (message or data or processor):
+            raise ValueError('You must pass either `message`, `data`, '
+                             'or `processor`')
         if timestamp is None:
             timestamp = time.time()
         self.buffer.append(({
-            'type': type,
+            'type': type or 'default',
             'timestamp': timestamp,
-            'duration': duration,
             'level': level,
             'message': message,
             'category': category,
@@ -39,9 +54,15 @@ class BreadcrumbBuffer(object):
         rv = []
         for idx, (payload, processor) in enumerate(self.buffer):
             if processor is not None:
-                processor(payload)
+                try:
+                    processor(payload)
+                except Exception:
+                    logger.exception('Failed to process breadcrumbs. Ignored')
+                    payload = None
                 self.buffer[idx] = (payload, None)
-            rv.append(payload)
+            if payload is not None and \
+               (not rv or not event_payload_considered_equal(rv[-1], payload)):
+                rv.append(payload)
         return rv
 
 
@@ -56,9 +77,14 @@ def make_buffer(enabled=True):
     return BlackholeBreadcrumbBuffer()
 
 
-def record_breadcrumb(type, timestamp=None, duration=None, level=None,
-                      message=None, category=None, data=None,
-                      processor=None):
+def record_breadcrumb(type, *args, **kwargs):
+    # Legacy alias
+    kwargs['type'] = type
+    return record(*args, **kwargs)
+
+
+def record(message=None, timestamp=None, level=None, category=None,
+           data=None, type=None, processor=None):
     """Records a breadcrumb for all active clients.  This is what integration
     code should use rather than invoking the `captureBreadcrumb` method
     on a specific client.
@@ -66,7 +92,7 @@ def record_breadcrumb(type, timestamp=None, duration=None, level=None,
     if timestamp is None:
         timestamp = time.time()
     for ctx in raven.context.get_active_contexts():
-        ctx.breadcrumbs.record(type, timestamp, duration, level, message,
+        ctx.breadcrumbs.record(type, timestamp, level, message,
                                category, data, processor)
 
 
@@ -78,9 +104,17 @@ def _record_log_breadcrumb(logger, level, msg, *args, **kwargs):
             return
 
     def processor(data):
-        formatted_msg = text_type(msg)
-        if args:
-            formatted_msg = msg % args
+        formatted_msg = msg
+
+        # If people log bad things, this can happen.  Then just don't do
+        # anything.
+        try:
+            formatted_msg = text_type(msg)
+            if args:
+                formatted_msg = msg % args
+        except Exception:
+            pass
+
         # We do not want to include exc_info as argument because it often
         # lies (set to a constant value like 1 or True) or even if it's a
         # tuple it will not be particularly useful for us as we cannot
@@ -92,7 +126,7 @@ def _record_log_breadcrumb(logger, level, msg, *args, **kwargs):
             'level': logging.getLevelName(level).lower(),
             'data': kwargs,
         })
-    record_breadcrumb('default', processor=processor)
+    record(processor=processor)
 
 
 def _wrap_logging_method(meth, level=None):
@@ -235,7 +269,7 @@ def _hook_requests():
 
     def send(self, request, *args, **kwargs):
         def _record_request(response):
-            record_breadcrumb('http', category='requests', data={
+            record(type='http', category='requests', data={
                 'url': request.url,
                 'method': request.method,
                 'status_code': response and response.status_code or None,
@@ -287,8 +321,7 @@ def _install_httplib():
             }
             data['data'].update(status)
             return data
-        record_breadcrumb('http', category='requests',
-                          processor=processor)
+        record(type='http', category='requests', processor=processor)
         return real_putrequest(self, method, url, *args, **kwargs)
 
     def getresponse(self, *args, **kwargs):
